#!/usr/bin/env ruby
#
# **********************************************************************
#  Copyright notice
#
#  (c) 2011-2017 Dominic Simm <dominic.simm@mpibpc.mpg.de>
#  All rights reserved
#
#  This file is part of Waggawagga.
#
#  Waggawagga is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 2 of the License, or
#  (at your option) any later version.
#
#  Waggawagga is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Waggawagga.  If not, see <http://www.gnu.org/licenses/>.
# **********************************************************************

# Changes 2011-2017 by Dominic Simm <dominic.simm@mpibpc.mpg.de>
# See the ChangeLog or git repository for details.

$AUTHOR       = 'dsimm'
$PROGRAM_NAME = 'waggawagga-cli'
$VERSION      = '1.0.0'

# This script is a command-line version of the RoR web-application Waggawagga.
# Main Functions:
# - Import protein sequences with SAH-predictions into local SQLite3-database
# - Computation of several analyses:
#   - Amino acid frequencies over all filtered SAHs
#   - Filtered table of SAHs with cutoff > 0.25 and length >= 14
#   - Length-Distribution of SAHs
#   - Gnuplot full-sequence figures of SAH-predictions
# - Export analysis results as CSV (Excel-sheets)
# - Bundling of found SAHs and rest

# System-Modules
require 'bio'
require 'csv'
require 'sqlite3'
require 'active_record'
require 'open-uri'

# Load Packages for OptionParser
require 'fileutils'
require 'optparse'
require 'optparse/time'
require 'ostruct'

# Establish database connection to SQLite3
# ActiveRecord::Base.logger = Logger.new('debug.log')
configuration = YAML::load(IO.read('config/database.yml'))
ActiveRecord::Base.establish_connection(configuration['development'])

# Workarounds: Dummy Rails class to simulate Rails environment
class Rails
	# Run modes:
	# - Pure:     ./waggawagga-cli.rb
	# - Packaged: ./lib/app/waggawagga-cli.rb
	@root = File.dirname(File.expand_path $0).gsub('/lib/app', '')
	class << self
		attr_accessor :root
	end
end

# Load necessary Rails models
load 'app/helpers/request_helper.rb'
load 'app/models/genome_evaluation.rb'
load 'app/models/coiled_coil_sequence.rb'
load 'app/models/shell_command.rb'
load 'app/models/svg.rb'
load 'app/models/graph.rb'

class EvaluationController # < ApplicationController

	# Global variables $
	$config_path = '%s/config' % [Rails.root]
	$tmp_path     = '%s/results' % [Rails.root]
	$tmp_path_data = '%s/results/sah_data' % [Rails.root]
	$tmp_path_image = '%s/results' % [Rails.root]

	# Load Scoring-Matrices
	$bottomLeftMatrix = RequestHelper.initialize_scoring_matrix("#{$config_path}/scoring_matrix_i_4.csv")
	$bottomRightMatrix = RequestHelper.initialize_scoring_matrix("#{$config_path}/scoring_matrix_i_3.csv")

	################################################################################################################
	#### Code Replacement ONLY from here on!
	################################################################################################################

	# Calculates the overlapping percentage of two provided sequence sections
	# => Result is relative to the first sequence
	# @param start0 [int] 		Start position of first sequence
	# @param end0 [int] 		  End position of first sequence
	# @param start1 [int] 		Start position of second sequence
	# @param end1 [int] 		  End position of second sequence
	# return float					  overlapping percentage
	def overlap(start0, end0, start1, end1)

		start0, end0 = greater(start0, end0)
		start1, end1 = greater(start1, end1)
		length0 = end0-start0
		length1 = end1-start1

		# No overlapping
		if end0 <= start1 || end1 <= start0
			#puts "Fall 1"
			return overlap = 0.0
		end
		if length0 >= length1
			# seq1 lies fully in seq0
			if start0 < start1 && end0 > end1
				#puts "Fall 2"
				overlap = length1/length0
				# seq1 starts in front of seq0
			elsif start0 > start1
				#puts "Fall 3"
				overlap = (end1-start0)/length0
				# seq1 ends behind seq0
			else
				#puts "Fall 4"
				overlap = (end0-start1)/length0
			end
		else
			# seq1 starts in front of seq0
			if start0 > start1 && end0 < end1
				#puts "Fall 5"
				overlap = 1.0
				# seq1 ends behind seq0
			elsif end0 > end1
				#puts "Fall 6"
				overlap = (end1-start0)/length0
			else
				#puts "Fall 7"
				overlap = (end0-start1)/length0
			end
		end
		return overlap

	end


	# Returns two integer in the right order
	# @param start0 [int] 		Start position
	# @param end0 [int] 		  End position
	def greater(start0, end0)
		if start0 > end0
			tmp = start0
			start0 = end0
			end0 = tmp
		end
		return start0.to_f, end0.to_f
	end


	# Returns quoted string
	# @param v [string] 		string to quote
	def sql_quote_string(v)
		if v.size > 0
			return v.to_s.gsub(/\\/, '\&\&').gsub(/'/, "''")
		else
			return v
		end
	end


	# Returns sanitize filename for file-system
	# @param filename [string] 		string to quote
	def sanitize_filename(filename)
		return filename.strip do |name|
			# NOTE: File.basename doesn't work right with Windows paths on Unix
			# get only the filename, not the whole path
			name.gsub!(/^.*(\\|\/)/, '')

			# Strip out the non-ascii character
			name.gsub!(/[^0-9A-Za-z.\-]/, '_')
		end
	end


	# Parse enriched FASTA headers for GeneOntology (GO) analysis
	# @param header [string] 	    FASTA header (full)
	# @param delimiter [string] 	Delimiter to split
	# @return nil
	def parse_fasta_header(header, sep = ' ')
		header_arr = header.split(sep)
		# Filter tuple data
		tuples = header_arr.map { |el| el if el.include?(':') }.compact
		# Map to hash
		hash_data = Hash[tuples.map { |el| el.split(':', 2) }]
	end


	# Checks and create user session directory if it not exists
	# @param dir [string] 	 User session directory
	def check_folder_structure(dir)
		# Check whether tmp-structure exists
		make_cmd = `mkdir #{$tmp_path_data}` if !File.exist?($tmp_path_data)
		session_dir = '%s/%s' % [$tmp_path, dir]
		if File.exist?(session_dir) == false
			print "- Create session-dir: '#{session_dir}'\n"
			make_tmp = `rm -rf #{session_dir} && mkdir #{session_dir}`
		else
			print "- Overwrite session-dir: '#{session_dir}'\n"
		end
	end


	# Filters the session directory into result and trash files
	# @param dir [string] 	 User session directory
	# @param files [string] 	 User session directory
	def filter_user_data(dir, files)
		# Check whether tmp-structure exists
		session_dir = '%s/%s' % [$tmp_path, dir]
		trashcan = '%s/%s/%s' % [$tmp_path, dir, 'trash']
		resultbin = '%s/%s/%s' % [$tmp_path, dir, 'result']
		if File.exist?(session_dir)
			print "- Filter data\n" # " in #{session_dir}\n"
			make_cmd = `mkdir #{$tmp_path_data}` if !File.exist?($tmp_path_data)
			make_cmd = `mkdir #{trashcan}` if !File.exist?(trashcan)
			make_cmd = `mkdir #{resultbin}` if !File.exist?(resultbin)
			# Breaks down with too much files
			# move_cmd = `mv #{session_dir}/*.txt #{trashcan}` if Dir.glob('%s/{*.txt}' % session_dir).size > 0
			# move_cmd = `mv #{session_dir}/*.svg #{trashcan}` if Dir.glob('%s/{*.svg}' % session_dir).size > 0
			# Unix: High data amount variant: find . -iname "*.txt" [...] | xargs -0 -I % mv % targetdir
			move_cmd = `cd #{session_dir} && find . -maxdepth 1 -iname "*.txt" -type f -print0 | xargs -0 -I % mv % trash` if Dir.glob('%s/{*.txt}' % session_dir).size > 0
			move_cmd = `cd #{session_dir} && find . -maxdepth 1 -iname "*.svg" -type f -print0 | xargs -0 -I % mv % trash` if Dir.glob('%s/{*.svg}' % session_dir).size > 0
			# Copy positive results
			files.each { |el|
				fileObj = el.split('/')
				copy_cmd = `cp #{trashcan}/#{fileObj[1]}.* #{resultbin}`
			}
			copy_cmd = `mv #{$tmp_path_data}/*.csv #{resultbin}`
		else
			print "- Note: Session-folder '%s' missing ...\n" % dir
		end
	end


	# Clean up trashcan in session directory
	# @param dir [string] 	 User session directory name (self-chosen)
	# @return bool
	def self.clean_up(dir)
		# Check whether tmp-structure exists
		session_dir = '%s/%s' % [$tmp_path, dir]
		trashcan    = '%s/%s' % [session_dir, 'trash']
		print "\nClean up\n"
		if File.exist?(trashcan) and !File.exist?("%s.tar.gz" % trashcan)
			print "- Packing trash: 'trash.tar.gz'\n"
			make_tar = `cd #{session_dir} && tar cfz trash.tar.gz trash`
			print "- Emptying session trash ...\n"
			rm_trash = `rm -rf #{trashcan}` # && mkdir #{trashcan}`
			return true
		else
			print "- No session trash available or trash-archive already existing.\n"
			return false
		end
	end


	# Shows the current progress in percentage
	# @param runner [int] 	   Counter, current state
	# @param maximum [int] 	   Maximum
	# @param timestamp [float] Timestamp
	# @return session_name
	def progress(runner, maximum, timestamp)
		now = Time.now
		if runner.to_f/maximum < 1.0
			STDOUT.print "\rProgress: %3.2f%% (%.2f sec.)" % [runner.to_f/maximum*100, now-timestamp]
			STDOUT.flush()
		else
			STDOUT.print "\rProgress: %3.2f%% (%.2f sec.)\n\n" % [runner.to_f/maximum*100, now-timestamp]
		end
	end


	# Checks and create user session directory if it not exists
	# @param genome [string] 	 	User session directory name (self-chosen)
	# @param fastaPath [string] 	Altenative user session directory name (filename)
	# @return session_name
	def self.session_name(genome, fastaPath)
		# return genome = !genome.empty? ? genome.to_s : '%s_%s' % [File.basename(fastaPath, '.*'), Time.now.to_s.slice(0..18).gsub(/:/,'-').gsub(/[ ]/,'_')]
		return genome = !genome.empty? ? genome.to_s : '%s' % [File.basename(fastaPath, '.*')]
	end


	# Checks whether gnuplot is installed on the system
	# @return [bool]
	def self.gnuplot_installed()
		path = `which gnuplot`
		version = `gnuplot --version`
		return !path.empty? && version.index(/^gnuplot/).eql?(0)
	end


	# Import a sequence dataset and calculate the sah_score for each sequence
	# @param genome     name of the sequence dataset (distinction)
	# @param fastaPath  absoulute filepath
	# @param gnuplot    Switch to control gnuplot
	def generate_sah_score(genome, fastaPath, gnuplot)
		windowSizes = [14,21,28,49]; min_seq_len = 49; segment_size = 0;
		tool = 'sah'; result = []; import_num = 0

		# Important notice for dataset
		genome = self.class.session_name(genome, fastaPath)
		check_folder_structure(genome)
		puts "- Dataset-ID: '%s'" % genome
		fastaPath = fastaPath.to_s || ''
 		puts "- FASTA-file: '%s'" % fastaPath

		# Abort if FastaFilename is empty
		Proc.new{ puts "Error: Please specify an existing FASTA file."; exit; }.call if fastaPath == '' || !File.exists?(fastaPath)
		multipleFasta = IO.read(fastaPath)

		# Initialize database-stuff
		cnt = 0; inserts = 0; sql_head = ''; sql_batch = []
		@CONN = ActiveRecord::Base.connection

		# Parse FASTA file into multiple sequences
		timestamp = Time.now
		sequences = Bio::Alignment::MultiFastaFormat.new(multipleFasta)
		seqs_num  = sequences.entries.size
		invalid   = sequences.entries.map { |el| el if el.seq.size < min_seq_len }.compact.size
		valid     = seqs_num-invalid
		total_num = valid*windowSizes.size
		puts "  # Found %d valid sequences to import (%d/%d sequences with len>=%daa)\n\n" % [valid, valid, seqs_num, min_seq_len]
		# exit(0)

		windowSizes.each_with_index { |size, i|
			$window_size = size
			sequences.entries.each_with_index do |entry, seq_num|
				if entry.seq.size >= min_seq_len
					# Check (strip/filter) Protein Sequence
					bioSeq = Bio::Sequence::AA.new(entry.seq)
					bioSeq.gsub!(/[^AC-IK-NP-TVWY]/, '')
					import_num += 1

					if $verbose
						puts "%d - Importing: %s; windowSize: %d" % [cnt, entry.entry_id, $window_size]
					else
						progress(import_num, total_num, timestamp)
					end
					# Create ShellCommand-object
					shell = ShellCommand.new($tmp_path, '', 0, size, bioSeq, entry.entry_id, 0)
					# Calculate SAH-scores and add it to result[]
					if $verbose
						STDOUT.print "- Analyzing ... "
					end
					# 'Extrapolate' shortened result data
					result = shell.extrapolate(tool, result)
					# Prepare data for SAH score and merge with existing tool output (last row)
					coiledcoil_sequence = shell.prepare_coildata(result, tool)
					# Calculate SAH window scores - 49 (default: $window_size) and 21, 13 for pure SAH
					shell.add_sah_score(coiledcoil_sequence, result)
					# Grab last col (SAH)
					scores = result.collect { |el| el[-1] }

					# Store result to disk - Systematic file-naming
					if $verbose
						puts "Writing to file: %s_%d_%s\n\n" % [import_num.to_s.rjust(total_num.to_s.length, "0"), size, entry.entry_id.gsub(/\W/, '_')]
					end
					filename = '%s_%d_%s' % [import_num.to_s.rjust(total_num.to_s.length, "0"), size, entry.entry_id.gsub(/\W/, '_')]
					# MD5-Hashing: Easy anti-duplicate file-names
					# filename = Digest::MD5.hexdigest("%s%s%d" % [tool, bioSeq, $window_size])
					imagePath = "%s/%s" % [genome, filename]
					shell.array_to_file(result, imagePath)

					# Load GNUplot-template, change it to cli-tool-format and execute
					if gnuplot
						shell.make_gnu(imagePath, shell.conf[:"#{tool}"][:gnuId], (shell.conf[:"#{tool}"][:rows].to_i+1).to_s, $window_size, shell.conf[:"#{tool}"][:max], coiledcoil_sequence, shell.conf[:"#{tool}"][:tics], tool)
					end

					# Due to performance issues of ActiveRecord, make inserts manually via direct SQL-import
					sql_head = 'INSERT INTO "genome_evaluations" ("genome", "entry_id", "header", "sequence", "scores", "imagePath", "windowSize", "segmentSize", "tool") VALUES '
					sql = "('%s', '%s', '%s', '%s', '%s', '%s', '%d', '%d', '%s')" % [genome, entry.entry_id, sql_quote_string(entry.definition), bioSeq, JSON.dump(scores), imagePath, size, segment_size, tool]
					sql_batch.push(sql)
					inserts += 1
				end
				# Write in chunks to database
				if inserts >= 50
					if $verbose
						puts "Execute SQL batch ..."
					end
					@CONN.execute sql_head + sql_batch.join(", ") + ";"
					sql_batch = []; inserts = 0
				end

				cnt += 1
			end

			# Write last results to database
			if inserts > 0
				if $verbose
					puts "Execute SQL batch ..."
				end
				@CONN.execute sql_head + sql_batch.join(", ") + ";"
				sql_batch = []; inserts = 0
			end
		}

		return cnt
	end


	# Makes on evaluation over all precalculated Genome-SAH-scores in the database
	# @param genome             name of the sequence dataset (distinction)
	# @param cutoff             SAH-ground-cutoff
	# @param domain_cutoffs     SAH-thresholds
	# @return void
	def evaluate_14avg_sah_genome_scores_by_length(genome, cutoff, domain_cutoffs = [0.34, 0.30, 0.30, 0.24])
		cnt = 0; windowSizes = [14,21,28,49]; average = 14; minSizeSahDomain = 14
		# Lower cutoff for domain cutting
		# puts 'Cutoff: %.2f' % cutoff # cutoff = 0.25;
		# Max-Average-Cutoff: Only sequences permitted with max score above domain-cutoff
		domain_cutoff = 0.0; # domain_cutoffs = [0.34, 0.30, 0.30, 0.24]
		# domain_cutoff = 0.0; domain_cutoffs = [0.50, 0.47, 0.42, 0.36];
		halfs = [(average.to_f/2).to_i, (average.to_f/2).ceil-1];

		windowSizes.each_with_index { |size, i|
			puts "Calculation for window-size: %d" % size
			domain_cutoff = domain_cutoffs[i]
			results = []; results_distinct = []
			sahScores = GenomeEvaluation.where(["\"genome\" = :g AND \"windowSize\" = :w", { :g => genome, :w => size }]).order("entry_id ASC")
			sahScores.each_with_index { |scoreObj, i|
				indeces = []; ranges = []; sequences = []; maximum = nil
				scores = JSON.load(scoreObj.scores)

				# Filter scores by cutoff
				inserts = 0; region_cnt = 0
				scores.each_with_index { |score, i|
					# Are we in next cutoff region of the scores
					if indeces.length > 1 && indeces.last-i < -1
						region_jump = true; region_cnt = 0; inserts = 0
					else
						region_jump = false
					end
					# Correct trailing lower cutoffs: In case of jump to next cutoff region
					while(indeces.length >= 5 && region_jump && scores[indeces.last].to_f < cutoff)
						indeces.pop
					end
					# Filter strictly by cutoff
					if score.to_f > cutoff
						indeces.push(i); region_cnt += 1
						# puts region_cnt.to_s + " " + (region_cnt.to_f*0.2).to_i.to_s
						# Permit also 1 outlier in 5aa: NO jump to next cutoff region and max. 20% lower inserts per SAH-domain (global)
					elsif indeces.length >= 5 && !region_jump && inserts <= (region_cnt.to_f*0.2).ceil # && scores[indeces[-5]].to_f > cutoff
						# Distribute lower inserts equally
						indeces.push(i); region_cnt += 1; inserts += 1
					end
				}

				# Do we have a result
				if indeces.length > 0
					# Convert filtered indeces to ranges
					ranges = indeces.sort.uniq.inject([]) do |spans, n|
						if spans.empty? || spans.last.last != n - 1
							spans + [n..n]
						else
							spans[0..-2] + [spans.last.first..n]
						end
					end

					# Select scores and sub-sequences
					ranges.each { |range|
						rangeScores = []
						# Permit only regions larger than 10aa
						if range.size >= minSizeSahDomain
							# Determine max average score
							range.each { |n|
								first = n-halfs[0] >= 0 ? n-halfs[0] : 0
								last  = n+halfs[1] <= scores.length ? n+halfs[1] : scores.length-1
								selectRange = Range.new(first, last)
								window = scores[selectRange]
								avgScore = 0
								window.each { |score| avgScore += score }
								avgScore /= average
								rangeScores.push(avgScore)
							}

							# Extend to average size
							preRange = Range.new(range.begin-4, range.begin+3)
							sufRange = Range.new(range.end-3, range.end+4)
							sequences.push([scoreObj.sequence[range], scores[range], rangeScores.max, Range.new(range.begin+1, range.end+1), scores[preRange], scores[sufRange]])  if rangeScores.max >= domain_cutoff
						end
					}

					# Results: Outcome for Cutoff-Filter table includes all found ranges with additional data
					sequences.each { |seq|
						# ID; Header; rangeScoreMax; Sequence[Range]; Range; Scores[Range]; Sequence; Taxonomy; Scores[preRange]; Scores[sufRange], classType; Sequence[Range].len; imagePath
						entry = [scoreObj.id, scoreObj.entry_id, seq[2], seq[0], seq[3], seq[1], scoreObj.sequence, scoreObj.taxonomy, seq[4], seq[5], scoreObj.class_type, seq[0].length, scoreObj.imagePath, scoreObj.header]
						results.push(entry)
						# Get maximum for results_distinct
						if !maximum.nil? && maximum[2] < seq[2]
							maximum = entry
						elsif maximum.nil?
							maximum = entry
						end
					}
					# Results: Outcome for Cutoff-Filter table includes all found ranges with additional data
					results_distinct.push(maximum) if !maximum.nil?
				end
			}

			puts "All Found SAHs:  " + results.size.to_s
			puts "Max SAH per seq: " + results_distinct.size.to_s
			# Sort results
			results.sort! { |x,y| y[2] <=> x[2]}            # DESC max14AvgScore
			results_distinct.sort! { |x,y| x[10] <=> y[10]} # ASC Class (Myosins)

			# Write sorted max values to ...txt
			tmp_filename_filter =  "sah_score_14avg_window_#{size}_cutoff_filter.csv"
			tmp_filename_length =  "sah_score_14avg_window_#{size}_length_distribution.csv"
			tmp_filename_aa_freq = "sah_score_14avg_window_#{size}_aa_freq.csv"

			# Length-Distribution of SAHs
			CSV.open("#{$tmp_path_data}/#{tmp_filename_length}", "w", {:col_sep => ";"}) do |csv|
				i = 0; results.sort! { |x,y| x[11] <=> y[11]}           # ASC SAH-seq-length
				# Insert Header + Data
				headers = ['ID','GeneID','Max14AvgScore','Length']
				csv << headers
				results.each { |result|
					csv << [(i+=1), result[1], result[2], result[11]]
				}
			end

			# Filtered table of SAHs with cutoff > 0.25 and length >= 14
			CSV.open("#{$tmp_path_data}/#{tmp_filename_filter}", "w", {:col_sep => ";"}) do |csv|
				i = 0; results.sort! { |x,y| y[2] <=> x[2]}             # DESC max14AvgScore
				# Insert Header + Data
				headers = ['ID','InternalID','GeneID','Header','GroundCutoff','MaxThreshold','Max14AvgScore','SahDomain','Length','Range','Scores[preRange(-4/+3)]','Scores[sufRange(-3/+4)]','Scores[Range]']
				csv << headers
				results.each { |result|
					csv << [(i+=1), result[0], result[1], result[13], cutoff, domain_cutoff, result[2], result[3], result[11], result[4], result[8], result[9], result[5]]
				}
			end

			# Amino acid frequencies over all filtered SAHs
			CSV.open("#{$tmp_path_data}/#{tmp_filename_aa_freq}", "w", {:col_sep => ";"}) do |csv|
				# Calculate nucleotide frequencies
				joined_seqs = results.map{ |entry| entry[3] }.join(''); aa_freq = []
				"ACDEFGHIKLMNPQRSTVWY".each_char { |c| aa_freq.push([c, joined_seqs.count(c)/joined_seqs.size.to_f]) }
				# "GPAVLIMCFYWHKRQNEDST".each_char { |c| aa_freq.push([c, joined_seqs.count(c)/joined_seqs.size.to_f]) }
				aa_freq.each { |result|
					csv << result
				}
			end

			# Filter interesting files (gnuplots, raw table data, sah-analysis data)
			files = results.map{ |entry| entry[12] }
			filter_user_data(genome, files)
		}
		# User acknowledgement
		puts "\nFind your filtered results in \n- %s/%s" % [$tmp_path, genome]

		return 0
	end

end


########################################################################################################################
## Main Script
########################################################################################################################

class Optparser

	ACTIONS = { "complete" => "complete", "import" => "import", "evaluation" => "evaluation", "example" => "example" }
	# ACTIONS = [ :complete, :import, :evaluation, :example ]
	WINDOW_SIZES = { "14" => 14, "21" => 21, "28" => 28, "49" => 49 }
	# Low and High Cutoffs for determine the "twilight zone"
	SDC_CUTOFF = { "14" => 0.34, "21" => 0.30, "28" => 0.30, "49" => 0.24 }
	# SDC_CUTOFF = { "14" => 0.50, "21" => 0.47, "28" => 0.42, "49" => 0.36 }


	# Helper method to provide reduced help-text
	def self.simple_help(program=$PROGRAM_NAME, version=$VERSION)
		help_text =
"%s %s (March 2017) - predicting single alpha-helices

FATAL: No sequence file(s) specified.

Usage: ./waggawagga-cli <FASTA file>

Basic Options:
    -h, --help                       Print full list of available options" % [program, version]
		print help_text
	end

	# Helper method to check actions
	def self.mandatory_check(actions, options)
		mandatory = actions                                                                 # Enforce the presence of
		missing = mandatory.select{ |param| options[param].nil? or options[param].empty? }  # the -a and -f switches
		unless missing.empty?                                                               #
			raise OptionParser::MissingArgument.new(missing.join(', '))                       #
		end
	end


	#
	# Return a structure describing the options.
	#
	def self.parse(args={})
		# The options specified on the command line will be collected in *options*.
		# We set default values here.
		options = OpenStruct.new
		options.action = ''                   # Action to preform
		options.file = ''                     # FASTA-data to import
		options.genome = ''                   # Identifier/Name for imported data
		options.gnuplot = false               # Switch to control GNUplotting
		options.threshold = 0.25              # SAH-Score threshold
		options.cutoff = 0.25                 # SAH-Domain-Score cutoff
		# options.sah_window_size = 49          # SAH-Score window-size
		# options.sah_domain_window_size = 49   # SAH-Domain-Score window-size
		options.verbose = false

		# Define options and arguments
		opt_parser = OptionParser.new do |opts|
			opts.program_name = $PROGRAM_NAME
			opts.release = $VERSION
			opts.version = Gem::Version.new($VERSION)
			opts.banner = "%s %s (March 2017) - predicting single alpha-helices" % [opts.program_name, opts.release]
			opts.separator "
Please cite:
Simm D., Hatje K. and Kollmar M. (2014) Waggawagga: comparative visualization
of coiled-coil predictions and detection of stable single α-helices (SAH domains).
Bioinformatics. 31(5):767-769. doi://10.1093/bioinformatics/btu700"
			opts.separator "
Waggawagga-cli uses a modified version of the original web application,
allowing user-specified scoring schemes for amino acid interactions."
# 			opts.separator "
# Simple Mode
# -----------
# Waggawagga can directly be used with the standard parameter-set only by specifying a FASTA-file:
#
# ➜ ./waggawagga-cli <'path/to/your/fastafile.fas'>
#
# For the 'Expert Mode', please have a look at the documentation.
# "
# Expert Mode
# -----------
# ➜ ./waggawagga-cli  -a <action> [options]"
# ##### TODO: Only for testing
# 			opts.separator "\nOnly for TESTING"
# 			opts.separator "Call: ./waggawagga-cli -a import -i 'dataset-id' -g -f 'path/to/your/fasta.fas'"
# 			opts.separator "Call: ./waggawagga-cli -a evaluation -i 'dataset-id' --tidy"
# 			opts.separator "Call: ./waggawagga-cli -a complete -g -f 'path/to/your/fasta.fas' --tidy"
# 			opts.separator "Call: ./waggawagga-cli -a example"
# #####
			opts.separator "\nUsage: ➜ ./waggawagga-cli -a <action> [options]"
			opts.separator ""
			opts.separator "Advanced options:"

			# Mandatory argument.
			opts.on("-a NAME", "--action NAME", ACTIONS, "Choose action (mandatory):", " (import, evaluation, complete, example)") do |a|
				options[:action] = a
			end

			# Mandatory argument.
			opts.on("-i NAME", "--id NAME", "Identifier for your imported dataset") do |i|
				options[:genome] = i
			end

			# Optional argument.
			opts.on("-f FILE", "--file FILE", "Input sequence-filepath", " (protein-sequences, Multiple FASTA format)") do |f|
				options[:file] = f
			end

			# # Optional argument with keyword completion.
			# opts.on("-w", "--window_size [SIZE]", WINDOW_SIZES,
			#         "Select the window-size for the SAH-Score (14,21,28,49)") do |w|
			# 	options.sah_window_size = w
			# end

			# # Optional argument with keyword completion.
			# opts.on("-d", "--domain_window_size [SIZE]", WINDOW_SIZES,
			#         "Select the window-size for the SAH-Domain-Score (14,21,28,49)") do |d|
			# 	options.sah_domain_window_size = d
			# end
			#
			# # Optional argument.
			# opts.on("-t", "--threshold [LIMIT]", Float, "Set threshold for max. average SAH-Domain-filtering (default=0.25)") do |t|
			# 	options.threshold = t
			# end

			# Optional argument.
			opts.on("-c LIMIT", "--cutoff LIMIT", Float, "Set SAH-Domain-Score ground-cutoff for the", "selection of SAH-Domains (default=0.25)") do |c|
				options[:cutoff] = c
			end

			# Boolean switch [optional]
			opts.on("-g", "--gnuplot", "Activate the generation of gnuplot figures") do |g|
				options[:gnuplot] = g
			end

			#######################################################
			# Standard options

			# Boolean switch [optional]
			opts.on("-t", "--tidy", "Store session trash files compressed") do |t|
				options[:tidy] = t
			end

			# Boolean switch [optional]
			opts.on("-v", "--verbose", "Run verbosely") do |v|
				options[:verbose] = v
			end

			# No argument, shows at tail.  This will print an options summary.
			# Try it and see!
			opts.on_tail("-h", "--help", "Print full list of available options") do
				puts opt_parser.help()
				exit
			end

			# Another typical switch to print the version.
			opts.on_tail("--version", "Show version") do
				puts opt_parser.version()
				exit
			end
		end

		# Handle enforced and unexpected requests
		begin
			opt_parser.parse!(args)
			Optparser.mandatory_check([:action], options)
			case options[:action]
				when 'import', 'complete'
					Optparser.mandatory_check([:file], options)
				when 'analysis'
					Optparser.mandatory_check([:genome], options)
			end
		# Show help on unspecified arguments
		rescue OptionParser::InvalidOption, OptionParser::InvalidArgument, OptionParser::MissingArgument => e
			puts "Warning: %s\n\n" % e
			puts opt_parser.help()
			exit
		end

		return options, opt_parser
	end  # parse()
end  # class Optparser



# Action: Import sequences into database
def import(options)
	# Start action
	puts "Import:"
	evaluation = EvaluationController.new
	gnuplot = options[:gnuplot] ? EvaluationController::gnuplot_installed() : false
	noi = evaluation.generate_sah_score(options[:genome], options[:file], gnuplot)
	puts "- Imported successfully into SQLite-database with 'dataset-id': \"%s\"" % options[:genome] if noi > 0
end

# Action: Analyse fix dataset
# @param options     OptionParser name of the sequence dataset (distinction)
# @return void
def evaluate(options)
	puts "\nEvaluation:"
	evaluation = EvaluationController.new
	evaluation.check_folder_structure(options[:genome])
	evaluation.evaluate_14avg_sah_genome_scores_by_length(options[:genome], options[:cutoff], Optparser::SDC_CUTOFF.values)
	tidy = options[:tidy] ? EvaluationController::clean_up(options[:genome]) : false
end

########################################################################################################################
## Process input
########################################################################################################################

# Special case 'Simple Mode': Only filename passed as argument - Mapping to OptionParser syntax
ARGV.prepend('-f').append('-a').append('complete').append('--tidy') if ARGV.size == 1 && (File.exists?('%s' % ARGV[0]) || File.exists?('%s/%s' % [Rails.root, ARGV[0]]))
# Parse input
if !ARGV.empty?
	options, opt_parser = Optparser.parse(ARGV)
else
	puts Optparser.simple_help()
	exit
end

# p ARGV
# p options
# p options[:threshold]

# Run selected action
$verbose = options[:verbose]
case options[:action]
	when 'import'
		# Parameter handling
		session_name = EvaluationController.session_name(options[:genome], options[:file])
		puts "\nThe dataset-id argument couldn't be recognized. Resuming now with: id = '%s'\n\n" % session_name if options[:genome].empty?
		options[:genome] = session_name
		import(options)
	when 'evaluation'
		evaluate(options)
	when 'complete'
		# Parameter handling
		options[:genome] = EvaluationController.session_name(options[:genome], options[:file])
		import(options)
		evaluate(options)
	when 'example'
		# Set default example parameter
		options[:file]   = 'test/example_myosin_seqs.fas'
		options[:genome] = 'example_myosin_seqs'
		import(options)
		evaluate(options)
	else
		# Default: Show help
		puts Optparser.simple_help()
		exit
end

